// URL of the CSV file for the Titanic dataset
const csvUrl =
  'https://gist.githubusercontent.com/saitejao07/89e2e36858c407517b15eb9d422f7117/raw/303a59c3d3a214f709f4b5484caf0d0f277310cf/titanic.csv';

// Load data from the CSV file using D3
d3.csv(csvUrl)
  .then(function (data) {
    // Filter data for survived passengers
    const survivedData = data.filter(
      (d) => d.Survived === '1',
    );

    // Filter data for non survived passengers
    const nonSurvivedData = data.filter(
      (d) => d.Survived === '0',
    );

    // Calculate the frequency of survivors by Pclass
    const survivedCounts = d3.rollup(
      survivedData,
      (v) => v.length,
      (d) => d.Pclass,
    );

    // Calculate the frequency of non-survivors by Pclass
    const nonSurvivedCounts = d3.rollup(
      nonSurvivedData,
      (v) => v.length,
      (d) => d.Pclass,
    );

    // Convert rollup map to array of objects for survived passengers
    const survivedCountsArray = Array.from(
      survivedCounts,
      ([key, value]) => ({ Pclass: key, Count: value }),
    );

    // Convert rollup map to array of objects for non-survived passengers
    const nonSurvivedCountsArray = Array.from(
      nonSurvivedCounts,
      ([key, value]) => ({ Pclass: key, Count: value }),
    );

    // Set up dimensions for the chart
    const width = 800;
    const height = 400;
    const margin = {
      top: 20,
      right: 20,
      bottom: 50,
      left: 50,
    };
    const innerWidth = width - margin.left - margin.right;
    const innerHeight = height - margin.top - margin.bottom;

    // Create SVG element
    const svg = d3
      .select('body')
      .append('svg')
      .attr('id', 'areachart') // Added id to select the SVG later
      .attr('width', width)
      .attr('height', height);

    // Set up color scale
    const color = d3
      .scaleOrdinal()
      .domain(['Survived', 'Non-Survived'])
      .range(['#1f77b4', '#ff7f0e']);

    // Create scales for survived passengers
    const xScale = d3
      .scaleBand()
      .domain(survivedCountsArray.map((d) => d.Pclass)) // Domain based on Pclass
      .range([margin.left, innerWidth])
      .padding(0.1);

    const yScale = d3
      .scaleLinear()
      .domain([
        0,
        d3.max(survivedCountsArray, (d) => d.Count),
      ]) // Domain based on the count of survivors
      .nice()
      .range([innerHeight, margin.top]);

    // Create area generator for survived passengers
    const area = d3
      .area()
      .x((d) => xScale(d.Pclass) + xScale.bandwidth() / 2)
      .y0(innerHeight)
      .y1((d) => yScale(d.Count));

    // Append area to SVG for survived passengers
    svg
      .append('path')
      .datum(survivedCountsArray)
      .attr('fill', color('Survived'))
      .attr('d', area);

    // Create scales for non-survived passengers
    const Non_xScale = d3
      .scaleBand()
      .domain(nonSurvivedCountsArray.map((d) => d.Pclass)) // Domain based on Pclass
      .range([margin.left, innerWidth])
      .padding(0.1);

    const Non_yScale = d3
      .scaleLinear()
      .domain([
        0,
        d3.max(nonSurvivedCountsArray, (d) => d.Count),
      ]) // Domain based on the count of non-survivors
      .nice()
      .range([innerHeight, margin.top]);

    // Create area generator for non-survived passengers
    const Non_area = d3
      .area()
      .x(
        (d) =>
          Non_xScale(d.Pclass) + Non_xScale.bandwidth() / 2,
      )
      .y0(innerHeight)
      .y1((d) => Non_yScale(d.Count));

    // Append area to SVG for non-survived passengers
    svg
      .append('path')
      .datum(nonSurvivedCountsArray)
      .attr('fill', color('Non-Survived'))
      .attr('d', Non_area);

    // Create legend for survived
    svg
      .append('rect')
      .attr('x', innerWidth - 80)
      .attr('y', 10)
      .attr('width', 20)
      .attr('height', 20)
      .style('fill', color('Survived'));

    svg
      .append('text')
      .attr('x', innerWidth - 50)
      .attr('y', 25)
      .text('Survived');

    // Create legend for non-survived
    svg
      .append('rect')
      .attr('x', innerWidth - 80)
      .attr('y', 40)
      .attr('width', 20)
      .attr('height', 20)
      .style('fill', color('Non-Survived'));

    svg
      .append('text')
      .attr('x', innerWidth - 50)
      .attr('y', 55)
      .text('Non-Survived');

    // Create axes
    const xAxis = d3.axisBottom(xScale);
    const yAxis = d3.axisLeft(yScale);

    // Append axes to SVG
    svg
      .append('g')
      .attr('transform', `translate(0, ${innerHeight})`)
      .call(xAxis)
      // X-axis label
      .append('text')
      .attr('x', innerWidth / 2)
      .attr('y', margin.bottom - 10)
      .attr('fill', '#000')
      .style('text-anchor', 'middle')
      .text('Passenger Class');

    svg
      .append('g')
      .attr('transform', `translate(${margin.left}, 0)`)
      .call(yAxis)
      // Y-axis label
      .append('text')
      .attr('transform', 'rotate(-90)')
      .attr('x', -innerHeight / 2)
      .attr('y', -margin.left + 15)
      .attr('fill', '#000')
      .style('text-anchor', 'middle')
      .text('Number of Survivors');
  })
  .catch(function (error) {
    console.log('Error loading data: ', error);
  });
